"""
Export table or query data to a text file.

Description:
This script exports a KexiDB table or query to a text file.

Author:
Sebastian Sauer <mail@dipe.org>

Copyright:
Dual-licensed under LGPL v2+higher and the BSD license.
"""

class Datasource:
	def __init__(self):
		import kexiapp
		keximainwindow = kexiapp.get("KexiAppMainWindow")

		try:
			self.connection = keximainwindow.getConnection()
		except:
			raise "No connection established. Please open a project before."

		self.schema = None

	def getSources(self):
		sources = []
		for table in self.connection.tableNames():
			sources.append("Tables/%s" % table)
		for query in self.connection.queryNames():
			sources.append("Queries/%s" % query)
		sources.sort()
		return sources

	def setSource(self, source):
		s = source.split("/",1)
		if s[0] == "Tables":
			self.schema = self.connection.tableSchema( s[1] )
			self.queryschema = self.schema.query()
		elif s[0] == "Queries":
			self.schema = self.connection.querySchema( s[1] )
			self.queryschema = self.schema
		self.cursor = None
		return self.schema != None

	def name(self):
		return self.schema.name()

	def caption(self):
		return self.schema.caption()

	def description(self):
		return self.schema.description()

	def header(self):
		h = []
		for field in self.schema.fieldlist().fields():
			s = field.caption()
			if s == None or s == "":
				s = field.name()
			h.append(s)
		return h

	def getNext(self):
		if not self.cursor:
			self.cursor = self.connection.executeQuerySchema( self.queryschema )
			if not self.cursor:
				raise "Failed to execute queryschema."
			if not self.cursor.moveFirst():
				raise "Failed to move cursor to first record."
		if self.cursor.eof():
			self.cursor = None
			return None
		items = []
		for i in range( self.cursor.fieldCount() ):
			items.append( self.cursor.value(i) )
		self.cursor.moveNext()
		return items

class TextExporter:
	def __init__(self, datasource, delimiter, encoding):
		self.datasource = datasource
		if delimiter == "Tabulator":
			self.delimiterBefore = ""
			self.delimiterAfter = "\t"
		elif delimiter == "Linebreak":
			self.delimiterBefore = ""
			self.delimiterAfter = "\n"
		else: # "Comma Separated Values"
			self.delimiterBefore = "\""
			self.delimiterAfter = "\","
		self.encoding = encoding

	def write(self, output):
		name = self.datasource.name()
		
		output.write("%s\n---\n" % name)

		#caption = self.datasource.caption()
		#if caption and caption != name:
		#	output.write("caption: %s\n" % caption)

		#description = self.datasource.description()
		#if description:
		#	output.write("description: %s\n" % description)

		#import datetime
		#output.write("date: %s\n---\n" % datetime.datetime.now())

		#header = self.datasource.header()
		while 1 == 1:
			items = self.datasource.getNext()
			if items == None: break
			output.write("\n")
			for item in items:
				if item != None and item != "":
					s = unicode(str(item),"latin-1").encode( self.encoding )
					output.write("%s%s%s" % (self.delimiterBefore,s,self.delimiterAfter))
		output.write("\n\n")

class GuiApp:
	def __init__(self, datasource):
		self.datasource = datasource

		try:
			import gui
		except:
			raise "Import of the Kross GUI module failed."

		self.dialog = gui.Dialog("Export Data to Text File")
                self.dialog.addLabel(self.dialog, "Export a table- or query-datasource to a Text File.")

		datasourceitems = self.datasource.getSources()
		self.datasourcelist = self.dialog.addList(self.dialog, "Datasource:", datasourceitems)

		self.fielddelimiterlist = self.dialog.addList(self.dialog, "Field Delimiter:", ("Comma Separated Values","Linebreak","Tabulator"))
		self.encodinglist = self.dialog.addList(self.dialog, "Text encoding:", ("utf-8","latin-1","ascii"))

		self.file = self.dialog.addFileChooser(self.dialog,
			"Text File:",
			gui.getHome() + "/kexidata.txt",
			(('Text files', '*.txt'),('All files', '*')))

		btnframe = self.dialog.addFrame(self.dialog)
		self.dialog.addButton(btnframe, "Export", self.doExport)
		self.dialog.addButton(btnframe, "Cancel", self.dialog.close)

		self.dialog.show()

	def doExport(self):
		file = str( self.file.get() )
		query = str( self.datasourcelist.get() )
		delimiter = str( self.fielddelimiterlist.get() )
		encoding = str( self.encodinglist.get() )
		if encoding == None or encoding == "":
			encoding = "utf-8"
		print "Exporting '%s' to file '%s' ..." % (query,file)

		if not self.datasource.setSource(query):
			raise "Invalid datasource selected."

		f = open(file, "w")
		global TextExporter
		exporter = TextExporter(self.datasource, delimiter, encoding)
		exporter.write(f)
		f.close()

		print "Successfully exported '%s' to file %s" % (query,file)
		self.dialog.close()

GuiApp( Datasource() )
